/* Fast-Clock millisecond timing module.  Operates by replacing 8253 clock  */
/* handler and increasing clock rate.  To use, call start clock before      */
/* timing.  Set ms_counter to zero when ready to time, and read value in    */
/* in milliseconds when done.  Always call stop_clock before terminating    */
/* program; this should be incorporated into and error handler.             */
/* Scott Cost, 1993                                                         */

#include <stdio.h>
#include <dos.h>
#include <conio.h>

																	 /* constants for working with 8253 timer */
#define Tics_per_MS 1192                   /* # tics per second, 8253 timer */
#define Timer_Set   0x34                                   /* control codes */
#define Timer_Reset 0x36
#define Timer_Vect  0x08                          /* clock interrupt vector */
#define T_Control   0x43                               /* 8253 control port */
#define T_Data      0x40                                  /* 8253 data port */

#ifdef __cplusplus
	#define __CPPARGS ...
#else
	#define __CPPARGS
#endif

void start_clock(__CPPARGS);
void stop_clock(__CPPARGS);
void interrupt fast_clock(__CPPARGS);          /* clock interrupt prototype */
void interrupt (*clock)(__CPPARGS);           /* original clock placeholder */

int ms_counter;                    /* global millisecond counter variable */

/* Upgrades system timing to millisecond accuracy.  Do not terminate        */
/* program under any circumstances without first stopping fast_clock, or    */
/* system will fail.  Time by zeroing and reading ms_counter.  Converted    */
/* from Mordkoff and Yantis' experimental suite, 1988.                      */
/* Scott Cost, 1993                                                         */
void start_clock(__CPPARGS)
{
	clock = getvect(Timer_Vect);           /* store i_vector for system clock */
	disable();                          /* disable interrupts before altering */
	setvect(Timer_Vect,fast_clock);   /* install fast_clock millisecond timer */
	outportb(T_Control,Timer_Set);                     /* set new clock speed */
	outportb(T_Data,Tics_per_MS & 0x00FF);                        /* low byte */
	outportb(T_Data,(Tics_per_MS & 0xFF00) / 0x0100);            /* high byte */
	enable();                                          /* re-enable interrupts*/
}

/* disables Fast-Clock millisecond timing mode.  See start_clock, above.    */
/* Scott Cost, 1993 */
void stop_clock(__CPPARGS)
{
	disable();                                          /* disable interrupts */
	setvect(Timer_Vect, clock);     /* restore original clock to timer vector */
	outportb(T_Control,Timer_Reset);                    /*  reset clock speed */
	outportb(T_Data,0x00);               /* reset to zero, low and high bytes */
	outportb(T_Data,0x00);
	enable();                                            /* enable interrupts */
}

/* Fast-Clock interrupt routine.  Supplants regular clock routine.          */
/* increments ms_counter once per millisecond.                              */
/* Scott Cost, 1993                                                         */
void interrupt fast_clock(__CPPARGS)
{
	_disable();                               /* no interrupts while handling */
	ms_counter++;                                        /* increment counter */
	outportb(0x20,0x20);                            /* send EOI to 8253 timer */
	enable();                                            /* enable interrupts */
}